# django imports
from django.contrib.auth.decorators import permission_required
from django.core.urlresolvers import reverse
from django.forms import ModelForm
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

from lfs.voucher.models import VoucherGroup

def manage_vouchers(request):
    """
    """
    try:
        voucher_group = VoucherGroup.objects.all()[0]
    except IndexError:
        url = reverse("lfs_manage_add_voucher_group")
    else:
        url = reverse("lfs_manage_voucher_group", kwargs={ "id" : voucher_group.id })
    
    return HttpResponseRedirect(url)
    
def add_voucher_group(request, template_name="manage/voucher/add_voucher_group.html"):
    """Adds a voucher group
    """
    if request.method == "POST":
        form = VoucherGroupForm(data=request.POST)
        if form.is_valid():
            voucher = form.save()
    else:
        form = VoucherGroupForm()
    
    return render_to_response(template_name, RequestContext(request, {
        "form" : form,
        "voucher_groups" : VoucherGroup.objects.all(),
    }))

def manage_voucher_group(request, id, template_name="manage/voucher/voucher_group.html"):
    """
    """
    voucher_group = VoucherGroup.objects.get(pk=id)    

    return render_to_response(template_name, RequestContext(request, {
        "form" : VoucherGroupForm(instance=voucher_group),
        "voucher_groups" : VoucherGroup.objects.all(),
    }))


# django imports
from django.contrib.auth.decorators import permission_required
from django.core.urlresolvers import reverse
from django.db import IntegrityError
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from django.utils import simplejson

# lfs imports
import lfs.voucher.utils
from lfs.core.utils import LazyEncoder
from lfs.core.utils import render_to_ajax_response
from lfs.tax.models import Tax
from lfs.voucher.forms import VoucherGroupForm
from lfs.voucher.models import Voucher
from lfs.voucher.models import VoucherGroup

# Parts
def voucher_group(request, id, template_name="manage/voucher/voucher_group.html"):
    """Main view to display a voucher group.
    """
    voucher_group = VoucherGroup.objects.get(pk=id)

    return render_to_response(template_name, RequestContext(request, {
        "voucher_group"  : voucher_group,
        "data_tab" : data_tab(request, voucher_group),
        "vouchers_tab" : vouchers_tab(request, voucher_group),
        "navigation" : navigation(request, voucher_group),
    }))

def navigation(request, voucher_group, template_name="manage/voucher/navigation.html"):
    """Displays the navigation.
    """
    return render_to_string(template_name, RequestContext(request, {
        "voucher_group" : voucher_group,
        "voucher_groups" : VoucherGroup.objects.all(),
    }))

def data_tab(request, voucher_group, template_name="manage/voucher/data.html"):
    """Displays the data tab of the passed voucher group.
    """
    if request.method == "POST":
        form = VoucherGroupForm(instance=voucher_group, data=request.POST)
        if form.is_valid():
            form = VoucherGroupForm(instance=voucher_group)
    else:
        form = VoucherGroupForm(instance=voucher_group)

    return render_to_string(template_name, RequestContext(request, {
        "voucher_group" : voucher_group,
        "form" : form,
    }))

def vouchers_tab(request, voucher_group, template_name="manage/voucher/vouchers.html"):
    """Displays the vouchers tab
    """
    vouchers = voucher_group.vouchers.all()
    taxes = Tax.objects.all()
    
    return render_to_string(template_name, RequestContext(request, {
        "voucher_group" : voucher_group,
        "vouchers" : vouchers,
        "taxes" : taxes,
    }))

# Actions
def manage_vouchers(request):
    """Redirects to the first voucher group or to the add voucher form.
    """
    try:
        voucher_group = VoucherGroup.objects.all()[0]
    except IndexError:
        url = reverse("lfs_manage_add_voucher_group")
    else:
        url = reverse("lfs_manage_voucher_group", kwargs={ "id" : voucher_group.id })

    return HttpResponseRedirect(url)

def add_vouchers(request, group_id):
    """
    """
    voucher_group = VoucherGroup.objects.get(pk=group_id)

    try:
        amount = int(request.POST.get("amount", 0))
    except TypeError:
        amount = 0

    for i in range(0, amount):
        while 1:
            try:
                Voucher.objects.create(
                    number = lfs.voucher.utils.create_voucher_number(),
                    group = voucher_group,
                    creator = request.user,
                    kind_of = request.POST.get("kind_of", 0),
                    value = request.POST.get("value", 0.0),
                    expiration_date = request.POST.get("expiration_date"),
                    tax_id = request.POST.get("tax_id")
                )
                break
            except IntegrityError:
                pass

    return render_to_ajax_response(
        (("#vouchers", vouchers_tab(request, voucher_group)), ),
        _(u"Vouchers have been created."))

def delete_vouchers(request, group_id):
    """Deletes checked vouchers.
    """
    voucher_group = VoucherGroup.objects.get(pk=group_id)
    vouchers = Voucher.objects.filter(pk__in=request.POST.getlist("voucher-ids"))

    for voucher in vouchers:
        voucher.delete()

    return render_to_ajax_response(
        (("#vouchers", vouchers_tab(request, voucher_group)), ),
        _(u"Vouchers have been deleted."))

def add_voucher_group(request, template_name="manage/voucher/add_voucher_group.html"):
    """Adds a voucher group
    """
    if request.method == "POST":
        form = VoucherGroupForm(data=request.POST)
        if form.is_valid():
            voucher_group = form.save(commit=False)
            voucher_group.creator = request.user
            voucher_group.save()
            url = reverse("lfs_manage_voucher_group", kwargs={"id" : voucher_group.id })
            return HttpResponseRedirect(url)
    else:
        form = VoucherGroupForm()

    return render_to_response(template_name, RequestContext(request, {
        "form" : form,
        "voucher_groups" : VoucherGroup.objects.all(),
    }))

def save_voucher_group_data(request, id):
    """Saves the data of the voucher group with passed id.
    """
    voucher_group = VoucherGroup.objects.get(pk=id)
    form = VoucherGroupForm(instance=voucher_group, data=request.POST)
    if form.is_valid():
        voucher_group = form.save()

    _update_positions()
    voucher_group = VoucherGroup.objects.get(pk=voucher_group.id)

    return render_to_ajax_response(
        (("#data_tab", data_tab(request, voucher_group)),
        ("#navigation", navigation(request, voucher_group)),),
        _(u"Voucher data has been save."))

def _update_positions():
    for i, voucher_group in enumerate(VoucherGroup.objects.all()):
        voucher_group.position = (i+1)*10
        voucher_group.save()


# django imports
from django.contrib.auth.models import User
from django.contrib.sessions.backends.file import SessionStore
from django.test import TestCase

# lfs imports
from lfs.cart.models import Cart
from lfs.cart.models import CartItem
from lfs.catalog.models import Product
from lfs.tests.utils import RequestFactory
from lfs.tax.models import Tax
from lfs.voucher.models import Voucher
from lfs.voucher.models import VoucherGroup
from lfs.voucher.settings import ABSOLUTE
from lfs.voucher.settings import PERCENTAGE

class CartModelsTestCase(TestCase):
    """
    """
    def setUp(self):
        """
        """
        self.tax = Tax.objects.create(rate=19.0)
        
        self.p1 = Product.objects.create(name="Product 1", slug="product-1", price=10.0, tax=self.tax)
        self.p2 = Product.objects.create(name="Product 2", slug="product-2", price=100.0, tax=self.tax)
        

        self.cart = Cart.objects.create()
        CartItem.objects.create(cart=self.cart, product=self.p1, amount=1)
        CartItem.objects.create(cart=self.cart, product=self.p2, amount=1)

    def test_get_price(self):
        """
        """
        price_net = self.cart.get_price_net()
        self.assertEqual("%.2f" % price_net, "%.2f" % 92.44)

    def test_get_gross(self):
        """
        """
        price_gross = self.cart.get_price_gross()
        self.assertEqual(price_gross, 110.0)
        
    def test_tax(self):
        """
        """
        tax = self.cart.get_tax()
        self.assertEqual("%.2f" % tax, "%.2f" % 17.56)
        
    def test_get_amount_of_items(self):
        """
        """
        amount = self.cart.amount_of_items
        self.assertEqual(2, 2)
        
    def test_get_items(self):
        """
        """
        items = self.cart.items()
        self.assertEqual(len(items), 2)


# django imports
from django.contrib.admin import widgets
from django.forms import ModelForm

# lfs imports
from lfs.voucher.models import VoucherGroup

class VoucherGroupForm(ModelForm):
    """Form to add a VoucherGroup.
    """
    class Meta:
        model = VoucherGroup
        fields = ("name", "position")


# python imports
import random
import datetime

# lfs imports
from lfs.voucher.models import Voucher

def create_voucher_number():
    """
    """
    letters = "ABCDEFGHIJKLMNOPQRSTUVXYZ"

    number = ""
    for i in range(0, 20):
        number += random.choice(letters)

    return number